@@ -19,7 +19,10 @@
 #ifndef COMMON_H
 #define COMMON_H 1
 
-/* #include <netinet/in.h> */	/* in_addr_t, in_port_t (TODO is standard?) */
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <inttypes.h>
 
 #define MAX_NICK 16                     /* 15 (+ '\0') */
