@@ -18,26 +18,32 @@
 
 #define _POSIX_SOURCE
 
+#if HAVE_CONFIG_H
+#include <config.h>
+#else
+#define HAVE_SYSINFO 1
+#define RETSIGTYPE void
+#endif
+
 #include <sys/time.h>		/* select */
 #include <sys/types.h>		/* select */
 #include <unistd.h>		/* select */
 #include <time.h>		/* time, localtime, mktime */
+#include <stdio.h>		/* stdlib.h (on some systems), printf, perror */
 #include <stdlib.h>		/* malloc, free */
 #include <errno.h>		/* errno */
 #include <signal.h>		/* signal */
-/* TODO if HAVE_SYSINFO */
-#if 0
+#if HAVE_SYSINFO
 #include <sys/sysinfo.h>	/* sysinfo */
 #endif
 #include "demux.h"
-#include "user_iface.h"
+#include "pqueue.h"		/* priority queue */
 
 /*
 #define DEMUX_DEBUG 1
 */
 
 #ifdef DEMUX_DEBUG
-# include <stdio.h>
 # define DEB(x) x
 # define DPRINTF(fmt, args...) printf( fmt, ## args )
 # define DPERROR(x) perror( x )
@@ -49,6 +55,7 @@
 
 extern int errno;
 
+/* Node in file descriptor lists. */
 struct fd_list_node {
 	int     fd;
 	void    (*callback) (int, void *);
@@ -57,24 +64,25 @@ struct fd_list_node {
 	struct fd_list_node *next;
 };
 
-struct timer_list_node {
-	timer_id id;
-	time_t  next_alarm;
+/* Node in timer queue. */
+struct timer_pq_node {
+	prio_t next_alarm;             /* must be the first field for pqueue */
 	unsigned long int time_incr;
 	long int time_shift;
-	void    (*callback) (void *);
-	void   *data;
-	struct timer_list_node *next;
+	void (*callback) (void *);
+	void *data;
 };
 
-/* The lists. */
+/* List of file descriptors. */
 static struct fd_list_node *input_list = 0, *output_list = 0;
-static struct timer_list_node *timer_list = 0;
+
+/* Timer queue (all fields automatically initialized to 0). */
+static pqueue timer_q;
 
 static volatile unsigned int recvd_signals = 0;
 
 #define MAX_SIGNALS (8 * sizeof( recvd_signals ))
-static void (*sig_handlers[MAX_SIGNALS]) (int);
+static RETSIGTYPE (*sig_handlers[MAX_SIGNALS]) (int);
 
 static int demux_enable;
 
@@ -85,23 +93,24 @@ static void fd_list_add (struct fd_list_node **list, int fd,
 static void * fd_list_remove (struct fd_list_node **list, int fd);
 static void fd_list_purge (struct fd_list_node **list);
 static void fd_list_cleanup (struct fd_list_node **list);
-static timer_id timer_list_add (time_t next_alarm,
-				unsigned long int time_incr,
-				long int time_shift,
-				void (*callback) (void *), void *data);
-static void timer_list_cleanup ();
-static struct timer_list_node *find_next_timer ();
+static timer_id timer_pq_add (time_t next_alarm,
+                              unsigned long int time_incr,
+                              long int time_shift,
+                              void (*callback) (void *),
+                              void *data);
+static void timer_pq_cleanup ();
+static struct timer_pq_node *find_next_timer ();
 static time_t time_quantize (unsigned long int seconds,
 			     unsigned long int shift);
-static void proc_timer (struct timer_list_node *timer);
-static void dmx_sighandler (int sig);
+static void proc_timer (struct timer_pq_node *timer);
+static RETSIGTYPE dmx_sighandler (int sig);
 static void dispatch_signals ();
 
 /* Begin demultiplexing. Doesn't return until dmx_stop() is called. */
 void
 dmx_main_loop ()
 {
-	struct timer_list_node *timer;
+	struct timer_pq_node *timer;
 	struct fd_list_node *node;
 	struct timeval tv, *ptv;
 	int     retval;
@@ -201,7 +210,7 @@ dmx_main_loop ()
 		else {
 			/* Error */
 			if (errno != EINTR) {
-				ui_perror ("select");
+				perror ("select");
 				demux_enable = 0;
 			}
 		}
@@ -220,7 +229,7 @@ dmx_cleanup ()
 {
 	fd_list_cleanup (&input_list);
 	fd_list_cleanup (&output_list);
-	timer_list_cleanup ();
+	timer_pq_cleanup();
 }
 
 /* Exit dmx_main_loop(). Registered fd's and timers remain. */
@@ -282,19 +291,19 @@ dmx_chk_output_fd (int fd)
    called until 'seconds' seconds have happened. */
 timer_id
 dmx_add_periodic_alarm (unsigned long int seconds, long int shift,
-			void (*callback) (void *), void *data)
+                        void (*callback) (void *), void *data)
 {
-	return timer_list_add (shift < 0 ? get_uptime () + seconds
-			       : time_quantize (seconds, shift),
-			       seconds, shift, callback, data);
+	return timer_pq_add (shift < 0 ? get_uptime() + seconds
+	                               : time_quantize(seconds, shift),
+	                     seconds, shift, callback, data);
 }
 
 /* Call 'callback' with 'data' as parameter in 'seconds' seconds from now. */
 timer_id
 dmx_add_timer (unsigned long int seconds, void (*callback) (void *),
-	       void *data)
+               void *data)
 {
-	return timer_list_add (get_uptime () + seconds, 0, 0, callback, data);
+	return timer_pq_add (get_uptime() + seconds, 0, 0, callback, data);
 }
 
 /* Check if timer needs to be dispatched and do it. Use when you are
@@ -302,16 +311,8 @@ dmx_add_timer (unsigned long int seconds, void (*callback) (void *),
 void
 dmx_dispatch_timer (timer_id id)
 {
-	struct timer_list_node *node;
-
-	for (node = timer_list; node; node = node->next) {
-		if (node->id == id) {
-			if (get_uptime () >= node->next_alarm) {
-				proc_timer (node);
-			}
-			break;
-		}
-	}
+	if (get_uptime() >= id->next_alarm)
+		proc_timer (id);
 }
 
 /* Call 'callback' when signal 'signum' arrives. This function must be used
@@ -352,22 +353,13 @@ dmx_remove_output_fd (int fd)
 void *
 dmx_remove_timer (timer_id id)
 {
-	struct timer_list_node *prev = (void *)0, *node;
-	void   *data = (void *)0;
+	struct timer_pq_node* node;
+	void *data = NULL;
 
-	for (node = timer_list; node; node = node->next) {
-		if (node->id == id) {
-			if (prev) {
-				prev->next = node->next;
-			}
-			else {
-				timer_list = node->next;
-			}
-			data = node->data;
-			free (node);
-			break;
-		}
-		prev = node;
+	node = (struct timer_pq_node *)pq_del_ptr (id, &timer_q);
+	if (node) {
+		data = node->data;
+		free (node);
 	}
 
 	return data;
@@ -387,19 +379,18 @@ dmx_remove_signal (int signum)
 static long
 get_uptime ()
 {
+#if (HAVE_SYSINFO && HAVE_STRUCT_SYSINFO_UPTIME)
+	struct sysinfo info;
+
+	sysinfo (&info);
+	return info.uptime;
+#else
 	/* TODO: This gives problems if system time changes.
 	 * timer_create() seems to have the same problem.
 	 * setitimer() maybe too.
 	 * sysinfo() is linux-only.
 	 * There is some good alternative??? */
 	return time (NULL);
-
-/* TODO if HAVE_SYSINFO */
-#if 0
-	struct sysinfo info;
-
-	sysinfo (&info);
-	return info.uptime;
 #endif
 }
 
@@ -470,65 +461,53 @@ fd_list_cleanup (struct fd_list_node **list)
 	*list = (void *)0;
 }
 
-/* Add a node to the timer list. */
-static  timer_id
-timer_list_add (time_t next_alarm, unsigned long int time_incr,
-		long int time_shift, void (*callback) (void *), void *data)
+/* Add a node to the timer queue. */
+static timer_id
+timer_pq_add (time_t next_alarm, unsigned long int time_incr,
+              long int time_shift, void (*callback) (void *),
+              void *data)
 {
-	struct timer_list_node *new_node;
+	struct timer_pq_node *new_node;
 
-	new_node = malloc (sizeof (struct timer_list_node));
-	new_node->id = timer_list ? timer_list->id + 1 : 0;
-	new_node->next_alarm = next_alarm;
+	if (timer_q.size == 0) {
+		/* not yet initialized */
+		if (pq_new (&timer_q, 127, NULL) < 0)
+			return NULL;
+	}
+
+	new_node = (struct timer_pq_node*)malloc (sizeof(struct timer_pq_node));
+	new_node->next_alarm = (prio_t)next_alarm;
 	new_node->time_incr = time_incr;
 	new_node->time_shift = time_shift;
 	new_node->callback = callback;
 	new_node->data = data;
-	new_node->next = timer_list;
-	timer_list = new_node;
 
-	return new_node->id;
+	pq_add (new_node, &timer_q);
+
+	return new_node;
 }
 
-/* Remove all nodes from the timer list. */
+/* Remove all nodes from the timer queue. */
 static void
-timer_list_cleanup ()
+timer_pq_cleanup ()
 {
-	struct timer_list_node *node, *next;
+	struct timer_pq_node *node;
 
-	for (node = timer_list; node; node = next) {
-		next = node->next;
+	while ((node = (struct timer_pq_node *)pq_del_min (&timer_q)))
 		free (node);
-	}
-	timer_list = (void *)0;
+
+	pq_delete (&timer_q);
 }
 
 /* Find next timer. */
-static struct timer_list_node *
+static struct timer_pq_node *
 find_next_timer ()
 {
-	struct timer_list_node *node, *min_node;
-	time_t  min_time;
-
-	if (timer_list) {
-		min_time = timer_list->next_alarm;
-		min_node = timer_list;
-		for (node = timer_list->next; node; node = node->next) {
-			if (node->next_alarm < min_time) {
-				min_time = node->next_alarm;
-				min_node = node;
-			}
-		}
-	}
-	else {
-		min_node = (void *)0;
-	}
-
-	return min_node;
+	return (struct timer_pq_node*)pq_min (&timer_q);
 }
 
 /* Quantize time in 'seconds' and return the time of the next quantization. */
-static  time_t
+static time_t
 time_quantize (unsigned long int seconds, unsigned long int shift)
 {
 	time_t  now = time (NULL);
@@ -547,48 +526,49 @@ time_quantize (unsigned long int seconds, unsigned long int shift)
 
 /* Process a timer. Call its callback and remove it or update it. */
 static void
-proc_timer (struct timer_list_node *timer)
+proc_timer (struct timer_pq_node *timer)
 {
-	time_t  last_alarm, now;
+	time_t  last_alarm, next_alarm, now;
 	void   *data = timer->data;
 	void    (*callback) (void *) = timer->callback;
+	struct timer_pq_node tmp_node;
 
 	if (timer->time_incr) {
-		last_alarm = timer->next_alarm;
+		last_alarm = next_alarm = timer->next_alarm;
 		if (timer->time_shift >= 0) {
-			timer->next_alarm =
-				time_quantize (timer->time_incr,
-					       timer->time_shift);
+			next_alarm = time_quantize (timer->time_incr,
+			                            timer->time_shift);
 		}
-		/* TODO: Tenemos un problema, una alarma periodica a una
-		   hora a veces se procesa por ejemplo en vez de a las
-		   11:00:00, a las 10:59:59, y luego se vuelve a procesar.
-		   De momento lo arreglo impidiendo dos alarmas en menos de
-		   dos segundos (Max). */
-#ifdef HI_RESOLUTION
-		if (timer->next_alarm == last_alarm) {
+		/* TODO: We've got a problem here. Sometimes a periodic
+		   alarm raises twice (eg. at 10:59:59 and 11:00:00).
+		   By now I fix it avoiding an alarm raising twice in less
+		   than two seconds (Max). */
+#ifdef DMX_HI_RESOLUTION
+		if (next_alarm == last_alarm) {
 #else
-		if (timer->next_alarm <= (last_alarm + 2)) {
+		if (next_alarm <= (last_alarm + 2)) {
 #endif
 			/* quantize gave the same time, or shift was
 			   -1, meaning quantization is not to be used */
-			timer->next_alarm += timer->time_incr;
+			next_alarm += timer->time_incr;
 			now = get_uptime ();
-			if (now > timer->next_alarm) {
+			if (now > next_alarm) {
 				/* maybe system time changed forward */
-				timer->next_alarm = now + timer->time_incr;
+				next_alarm = now + timer->time_incr;
 			}
 		}
+		/* this updates timer->next_alarm */
+		pq_chpri_ptr (timer, (prio_t)next_alarm, &timer_q);
 	}
 	else {
-		dmx_remove_timer (timer->id);
+		dmx_remove_timer (timer);
 	}
 
 	callback (data);
 }
 
 /* Handler for all signals. */
-static void
+static RETSIGTYPE
 dmx_sighandler (int sig)
 {
 	recvd_signals |= (1 << sig);
@@ -599,7 +579,7 @@ static void
 dispatch_signals ()
 {
 	unsigned int s;
-	int     i;
+	int          i;
 
 	while (recvd_signals) {
 		DPRINTF ("demux: signal catched\n");
