@@ -23,6 +23,7 @@
 
 #include <ctype.h>  
 #include <pwd.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -30,7 +31,6 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
-#include <stdio.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -67,15 +67,45 @@ home_path (char *file)
 error:
 	return NULL;
 }
+
 	
+char *
+home_path2 (char *dir, char *file)
+{
+	struct passwd *pass;
+	size_t size;
+	char *buf;
+	
+	pass = getpwuid (getuid());
+	CHECK (pass != NULL);
+				
+	size = strlen (pass->pw_dir);
+	buf = malloc ((size + strlen(dir) + strlen(file) + 2) * sizeof (char));
+	CHECK (buf != NULL);
+
+	strcpy (buf, pass->pw_dir);
+	
+	buf [size] = '/';
+	buf [size + 1] = '\0';
+
+	strcat (buf, dir);
+	strcat (buf, file);
+
+	return buf;
+
+error:
+	return NULL;
+}
+
+
 int
-check_dir ()
+check_dir (char *dirname)
 {
 	struct stat str;
 	char *dir;	
 	int ret;
 	
-	dir = home_path (".ipchat");
+	dir = home_path (dirname);
 	CHECK_J (dir != NULL, error_r);
 	
 	if (stat (dir, &str) == 0) {
@@ -361,6 +391,8 @@ write_tag (int fd, char *name, char *buff, size_t tabs)
 		strcat (b, str6); 
 	}
 	
+	free (t);
+
 	r = write (fd, b, strlen (b));
 	if (r != strlen (b)) {
 		free (b);
