@@ -29,9 +29,10 @@
 #define ERR_CO (ERR_LOCAL -5)
 
 extern char *home_path (char *file);
+extern char *home_path2 (char *dir, char *file);
 extern int read_tag (int fd, char *src, char *name, char *buff, size_t size);
 extern int write_tag (int fd, char *name, char *buff, size_t tabs);
-extern int check_dir ();
+extern int check_dir (char *dirname);
 extern char *ipv4_to_string (ip_t ip);
 
 #endif
