@@ -19,11 +19,11 @@
 #define _POSIX_SOURCE
 #define _ISOC99_SOURCE
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
-#include <stdio.h>
 #include <ctype.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -126,7 +126,7 @@ pa_parse_line (contact_t *win_contact, const char *line)
 				ct->state.in_buf_rcvd=0;
 				ct->state.out_buffer=NULL;
 				ct->state.out_buf_size=0;
-				ct->state.hello_timeout=(timer_id)-1;
+				ct->state.hello_timeout=NULL;
 				ct->state.ui_info=NULL;
 				cl_add_contact (ct);
 			}
@@ -286,7 +286,7 @@ command_id (char *str, char (*cl)[MAX_L], int num)
 	int ret = ERR_UNK;
 	char *c;
 
-	c = malloc (strlen (str) * sizeof (char));
+	c = malloc ((strlen (str) + 1) * sizeof (char));
 	if (c == NULL)
 		return ERR;
 
@@ -438,8 +438,7 @@ set_help (int var_id)
 			ui_output_info ("");
 			ui_output_info ("SET LOGGING [ON|OFF]");
 			ui_output_info ("");
-			ui_output_info ("Activates/deactivates logging "
-			                "(NOT YET IMPLEMENTED).");
+			ui_output_info ("Activates/deactivates logging.");
 			break;
 		case TIMESTAMP_V:
  			ui_output_info ("");
@@ -518,8 +517,7 @@ set_command (char *var, char *p)
 
 			u = TRUE;
 			if (cfg.logging)
-				ui_output_info ("\\blogging = ON (NOT YET "
-				                "IMPLEMENTED)");
+				ui_output_info ("\\blogging = ON");
 			else
 				ui_output_info ("\\blogging = OFF");
 		}
