@@ -31,6 +31,9 @@
 typedef struct {
 	char    nick[MAX_NICK];
 	port_t  listen_port;
+	BOOL	beep;
+	BOOL	logging;
+	BOOL	timestamps;
 } config_t;
 
 extern config_t cfg;
