@@ -18,9 +18,13 @@
 
 #define _POSIX_SOURCE
 
+#include <stdio.h>		/* stdlib.h (on some systems) */
 #include <stdlib.h>		/* malloc, free */
 #include <string.h>		/* memcpy, memset */
 #include <time.h>		/* time, strftime, localtime */
+#if TM_IN_SYS_TIME
+#include <sys/time.h>		/* struct tm */
+#endif
 #include "ui_common.h"
 #include "uconfig.h"
 
@@ -386,13 +390,13 @@ format_waddnstr (WINDOW *win, const char *str, int n, BOOL ps)
 		i = 0;
 		if (fmt) {
 			/* search sequence or new line */
-			while (str[i] != '\n' && str[i] != SEQCH && i < n)
+			while (i < n && str[i] != '\n' && str[i] != SEQCH)
 				i++;
 		}
 		else {
 			/* accelerated search when no format is active
 			   (that's most of the time) */
-			while (str[i] != SEQCH && i < n)
+			while (i < n && str[i] != SEQCH)
 				i++;
 		}
 
@@ -485,7 +489,13 @@ get_timestamp ()
 	static char tstamp[TSTAMP_LEN + 1];
 	time_t t = time (NULL);
 
+#if HAVE_STRFTIME
 	strftime (tstamp, TSTAMP_LEN + 1, "%H:%M", localtime (&t));
+#else
+	struct tm *ptm = localtime (&t);
+	snprintf (tstamp, TSTAMP_LEN + 1, "%02d:%02d", ptm->tm_hour,
+	          ptm->tm_min);
+#endif
 
 	return tstamp;
 }
